<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Console\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Class RestProcess
 * @package FiloBlu\MappConnector\Console\Command
 */
class RestProcess extends Command
{

    /**
     * @var \FiloBlu\MappConnector\Model\Jobs
     */
    protected $_jobs;

    /**
     * @var \FiloBlu\MappConnector\Model\Mapp
     */
    protected $_mapp;

    /**
     * @var \FiloBlu\MappConnector\Helper\FileLoggerHelper
     */
    protected $_loggerHelper;

    /**
     * @var \Magento\Framework\App\State
     */
    protected $_state;

    /**
     * @var array
     */
    protected $_entities = ['customer','subscriber'];

    /**
     * RestProcess constructor.
     * @param \FiloBlu\MappConnector\Model\Mapp $mapp
     * @param \FiloBlu\MappConnector\Model\Jobs $jobs
     * @param \Magento\Framework\App\State $state
     * @param \FiloBlu\MappConnector\Helper\FileLoggerHelper $loggerHelper
     */
    public function __construct(
        \FiloBlu\MappConnector\Model\Mapp $mapp,
        \FiloBlu\MappConnector\Model\Jobs $jobs,
        \Magento\Framework\App\State $state,
        \FiloBlu\MappConnector\Helper\FileLoggerHelper $loggerHelper
    )
    {
        $this->_jobs = $jobs;
        $this->_mapp = $mapp;
        $this->_state = $state;
        $this->_loggerHelper = $loggerHelper;
        parent::__construct();
    }

    /**
     * {@inheritdoc}
     */
    protected function execute(
        InputInterface $input,
        OutputInterface $output
    )
    {
        //Area Code Fix
        try {
            $this->_state->setAreaCode(\Magento\Framework\App\Area::AREA_GLOBAL);
        }
        catch (\Exception $e) {
            //Area code is already set
        }

        $entity = strtolower($input->getArgument('entity'));

        $result = true;

        $result = $this->processRestJobs($entity);

        if(!$result){
            $output->writeln("WARNING: The entity specified ".$entity." doesn't exists");
        }

    }

    public function processRestJobs($entity = null){
        if(!empty($entity)) {
            if(in_array($entity, $this->_entities)) {
                //Get all rest jobs in waiting status
                $jobs = $this->_jobs
                    ->getCollection()
                    ->addFieldToFilter('status', 'waiting')
                    ->addFieldToFilter('mode', 'rest')
                    ->addFieldToFilter('entity', $entity)
                    ->getData();
            }
            else {
                return false;
            }
        }
        else {
            //Get all rest jobs in waiting status
            $jobs = $this->_jobs
                ->getCollection()
                ->addFieldToFilter('status', 'waiting')
                ->addFieldToFilter('mode', 'rest')
                ->getData();
        }

        if(count($jobs)){
            foreach ($jobs as $j)
            {
                $this->_loggerHelper->debug('');
                $this->_loggerHelper->debug('-------- Start to process '.ucfirst($j['entity']).' '.$j['entity_id'].' --------');
                //Default operation type Create or Update
                $this->_mapp->setJob($j);
                //Make request update job and reset log
                $result = $this->_jobs->updateJobAndResetLog($j,$this->_mapp->getRequest(),$this->_mapp->makeRequest());
                $this->_loggerHelper->debug($result);
            }
        }
        else {
            $this->_loggerHelper->debug('-------- There are no record to process --------');
        }
    }

    /**
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this->setName("mapp:rest:process");
        $this->setDescription("Process waiting Mapp REST jobs");
        $this->addArgument('entity', InputArgument::OPTIONAL, __('Please specify an Entity'));
        parent::configure();
    }
}
