<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model\From\Entity;

/**
 * Class Subscriber
 * @package FiloBlu\MappConnector\Model\From\Entity
 */
class Subscriber extends \FiloBlu\MappConnector\Model\From\Entity\AbstractEntity implements \FiloBlu\MappConnector\Model\From\Entity\EntityInterface   {

    /**
     * Entity
     *
     * @var string
     */
    protected $_entity = 'subscriber';

    /**
     * Magento Entity Model
     *
     * @var string
     */
    protected $_entityModel = '\Magento\Newsletter\Model\Subscriber';

    /**
     * Magento Entity Model Id
     *
     * @var string
     */
    protected $_entityModelPrimary = 'subscriber_id';

    /**
     * Magento Entity Model Related Id
     *
     * @var string
     */
    protected $_entityModelRelatedPrimary = 'customer_id';

    /**
     * Mapp Resource Model
     *
     * @var string
     */
    protected $_entityMappResource = '\FiloBlu\MappConnector\Model\To\Mapp\User';

    /**
     * Mapp Resource Model Id
     *
     * @var array
     */
    protected $_entityMappPrimary = ['primaryDB' => 'subscriber_email', 'primaryMapp' => 'email'];

    /**
     * User Type
     *
     * @var string
     */
    protected $_userType = 'guest';

    /**
     * @var string
     */
    protected $_xmlConfigPath = 'mappconnector/subscriber_mapping/subscriber';

    /**
     * @var Customer
     */
    protected $_customer;

    /**
     * @var \FiloBlu\MappConnector\Model\Config
     */
    protected $_config;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Framework\Module\Manager
     */
    protected $_moduleManager;

    /**
     * @var \Magento\Framework\App\ResourceConnection
     */
    protected $_resourceConnection;

    /**
     * @var \Magento\Framework\Event\Manager
     */
    protected $_eventManager;

    /**
     * Subscriber constructor.
     * @param Customer $customer
     */
    public function __construct(
        \FiloBlu\MappConnector\Model\From\Entity\Customer $customer,
        \FiloBlu\MappConnector\Model\Config $config,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Module\Manager $moduleManager,
        \Magento\Framework\App\ResourceConnection $resourceConnection,
        \Magento\Framework\Event\Manager $eventManager
    )
    {
        $this->_customer = $customer;
        $this->_config = $config;
        $this->_storeManager = $storeManager;
        $this->_moduleManager = $moduleManager;
        $this->_resourceConnection = $resourceConnection;
        $this->_eventManager = $eventManager;
    }

    /**
     * @return {@inheritDoc}
     */
    public function getDefaultData($record){
        $result = [];
        $result['store_code'] = $this->_storeManager->getStore($record['store_id'])->getCode();
        $result['action'] = 'create';
        $result['subscriber_status_normalized'] = ($record['subscriber_status'] == 1)?"1":"0";
        return $result;
    }

    /**
     * @return {@inheritDoc}
     */
    public function getObjectData($primary,$id)
    {
        $connection = $this->_resourceConnection->getConnection();
        $tableName = $this->_resourceConnection->getTableName('newsletter_subscriber'); //gives table name with prefix

        //Select Data from table
        $sql = "SELECT * FROM " . $tableName.' WHERE '.$primary.' = '.$id;
        $result = $connection->fetchAll($sql);

        return $result[0];
    }

    /**
     * @return mixed
     */
    public function getAllData($type = 'full', $last_index = '')
    {
        $connection = $this->_resourceConnection->getConnection();
        $tableName = $this->_resourceConnection->getTableName('newsletter_subscriber'); //gives table name with prefix
        $tableNameJobs = $this->_resourceConnection->getTableName('filoblu_mappconnector_jobs');

        if($type == 'full') {
            //Full
            $sql = "Select * FROM " . $tableName;
            $result = $connection->fetchAll($sql);
            return $result;
        }
        else {
            //Incremental
            $sql = "SELECT 
                      ce.subscriber_id as subscriber_id
                    FROM
                      ".$tableName." AS ce
                    LEFT JOIN
                      ".$tableNameJobs." AS mj 
                    ON 
                      mj.entity_id = ce.subscriber_id
                    AND 
                      mj.entity = 'subscriber'
                    WHERE 
                      mj.status is null";

            return $connection->fetchAll($sql);
        }
    }

    /**
     * @param $record
     * @return array
     */
    public function getAdditionalData($record){
        $result = [];

        $result = array_merge(
            $result,
            $this->getRelatedModelData($this->_customer,$record['customer_id'])
        );

        $result = array_merge($result,['subscriberid' => $record['subscriber_id']]);

        // Dispatch event for newsletter observer
        $subscriber_data = new \Magento\Framework\DataObject(array('subscriber_data' => $result));
        $this->_eventManager->dispatch('mappconnector_subscriber_additional_data',['result'=>$subscriber_data]);
        $result = $subscriber_data->getSubscriberData();

        return $result;
    }

}